@@ -20,9 +20,53 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <gee/readonlyset.h>
+#include <glib.h>
+#include <glib-object.h>
 
 
+#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
+#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
+#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
+#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))
+
+typedef struct _GeeIterable GeeIterable;
+typedef struct _GeeIterableIface GeeIterableIface;
+
+#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
+#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
+#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
+#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))
+
+typedef struct _GeeIterator GeeIterator;
+typedef struct _GeeIteratorIface GeeIteratorIface;
+
+#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
+#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
+#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
+#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))
+
+typedef struct _GeeCollection GeeCollection;
+typedef struct _GeeCollectionIface GeeCollectionIface;
+
+#define GEE_TYPE_SET (gee_set_get_type ())
+#define GEE_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_SET, GeeSet))
+#define GEE_IS_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_SET))
+#define GEE_SET_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_SET, GeeSetIface))
+
+typedef struct _GeeSet GeeSet;
+typedef struct _GeeSetIface GeeSetIface;
+
+#define GEE_TYPE_READ_ONLY_SET (gee_read_only_set_get_type ())
+#define GEE_READ_ONLY_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySet))
+#define GEE_READ_ONLY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySetClass))
+#define GEE_IS_READ_ONLY_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_SET))
+#define GEE_IS_READ_ONLY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_SET))
+#define GEE_READ_ONLY_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySetClass))
+
+typedef struct _GeeReadOnlySet GeeReadOnlySet;
+typedef struct _GeeReadOnlySetClass GeeReadOnlySetClass;
+typedef struct _GeeReadOnlySetPrivate GeeReadOnlySetPrivate;
+
 #define GEE_READ_ONLY_SET_TYPE_ITERATOR (gee_read_only_set_iterator_get_type ())
 #define GEE_READ_ONLY_SET_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_READ_ONLY_SET_TYPE_ITERATOR, GeeReadOnlySetIterator))
 #define GEE_READ_ONLY_SET_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_READ_ONLY_SET_TYPE_ITERATOR, GeeReadOnlySetIteratorClass))
@@ -34,6 +78,71 @@ typedef struct _GeeReadOnlySetIterator GeeReadOnlySetIterator;
 typedef struct _GeeReadOnlySetIteratorClass GeeReadOnlySetIteratorClass;
 typedef struct _GeeReadOnlySetIteratorPrivate GeeReadOnlySetIteratorPrivate;
 
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIteratorIface {
+	GTypeInterface parent_iface;
+	gboolean (*next) (GeeIterator* self);
+	gpointer (*get) (GeeIterator* self);
+};
+
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIterableIface {
+	GTypeInterface parent_iface;
+	GeeIterator* (*iterator) (GeeIterable* self);
+	GType (*get_element_type) (GeeIterable* self);
+};
+
+/**
+ * Serves as the base interface for implementing collection classes. Defines
+ * size, iteration, and modification methods.
+ */
+struct _GeeCollectionIface {
+	GTypeInterface parent_iface;
+	gboolean (*contains) (GeeCollection* self, gconstpointer item);
+	gboolean (*add) (GeeCollection* self, gconstpointer item);
+	gboolean (*remove) (GeeCollection* self, gconstpointer item);
+	void (*clear) (GeeCollection* self);
+	gboolean (*add_all) (GeeCollection* self, GeeCollection* collection);
+	gboolean (*contains_all) (GeeCollection* self, GeeCollection* collection);
+	gboolean (*remove_all) (GeeCollection* self, GeeCollection* collection);
+	gboolean (*retain_all) (GeeCollection* self, GeeCollection* collection);
+	gpointer* (*to_array) (GeeCollection* self, int* result_length1);
+	gint (*get_size) (GeeCollection* self);
+	gboolean (*get_is_empty) (GeeCollection* self);
+};
+
+/**
+ * A set is a collection without duplicates.
+ */
+struct _GeeSetIface {
+	GTypeInterface parent_iface;
+};
+
+/**
+ * Represents a read-only collection of items without duplicates.
+ */
+struct _GeeReadOnlySet {
+	GObject parent_instance;
+	GeeReadOnlySetPrivate * priv;
+};
+
+struct _GeeReadOnlySetClass {
+	GObjectClass parent_class;
+};
+
+struct _GeeReadOnlySetPrivate {
+	GType g_type;
+	GBoxedCopyFunc g_dup_func;
+	GDestroyNotify g_destroy_func;
+	GeeSet* _set;
+};
+
 struct _GeeReadOnlySetIterator {
 	GObject parent_instance;
 	GeeReadOnlySetIteratorPrivate * priv;
@@ -43,36 +152,59 @@ struct _GeeReadOnlySetIteratorClass {
 	GObjectClass parent_class;
 };
 
-
-
-struct _GeeReadOnlySetPrivate {
-	GeeSet* _set;
-	GType g_type;
-	GBoxedCopyFunc g_dup_func;
-	GDestroyNotify g_destroy_func;
-};
-
-#define GEE_READ_ONLY_SET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySetPrivate))
-enum  {
-	GEE_READ_ONLY_SET_DUMMY_PROPERTY,
-	GEE_READ_ONLY_SET_SIZE,
-	GEE_READ_ONLY_SET_SET,
-	GEE_READ_ONLY_SET_G_TYPE,
-	GEE_READ_ONLY_SET_G_DUP_FUNC,
-	GEE_READ_ONLY_SET_G_DESTROY_FUNC
-};
-static GType gee_read_only_set_real_get_element_type (GeeIterable* base);
-static GeeIterator* gee_read_only_set_real_iterator (GeeIterable* base);
-static gboolean gee_read_only_set_real_contains (GeeCollection* base, gconstpointer item);
-static gboolean gee_read_only_set_real_add (GeeCollection* base, gconstpointer item);
-static gboolean gee_read_only_set_real_remove (GeeCollection* base, gconstpointer item);
-static void gee_read_only_set_real_clear (GeeCollection* base);
 struct _GeeReadOnlySetIteratorPrivate {
 	GType g_type;
 	GBoxedCopyFunc g_dup_func;
 	GDestroyNotify g_destroy_func;
 };
 
+
+static gpointer gee_read_only_set_iterator_parent_class = NULL;
+static GeeIteratorIface* gee_read_only_set_iterator_gee_iterator_parent_iface = NULL;
+static gpointer gee_read_only_set_parent_class = NULL;
+static GeeIterableIface* gee_read_only_set_gee_iterable_parent_iface = NULL;
+static GeeCollectionIface* gee_read_only_set_gee_collection_parent_iface = NULL;
+static GeeSetIface* gee_read_only_set_gee_set_parent_iface = NULL;
+
+GType gee_iterator_get_type (void);
+GType gee_iterable_get_type (void);
+GType gee_collection_get_type (void);
+GType gee_set_get_type (void);
+GType gee_read_only_set_get_type (void);
+#define GEE_READ_ONLY_SET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySetPrivate))
+enum  {
+	GEE_READ_ONLY_SET_DUMMY_PROPERTY,
+	GEE_READ_ONLY_SET_SIZE,
+	GEE_READ_ONLY_SET_IS_EMPTY,
+	GEE_READ_ONLY_SET_SET,
+	GEE_READ_ONLY_SET_ELEMENT_TYPE,
+	GEE_READ_ONLY_SET_G_TYPE,
+	GEE_READ_ONLY_SET_G_DUP_FUNC,
+	GEE_READ_ONLY_SET_G_DESTROY_FUNC
+};
+static void gee_read_only_set_set_set (GeeReadOnlySet* self, GeeSet* value);
+GeeReadOnlySet* gee_read_only_set_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeSet* set);
+GeeReadOnlySet* gee_read_only_set_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeSet* set);
+static GeeReadOnlySetIterator* gee_read_only_set_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
+static GeeReadOnlySetIterator* gee_read_only_set_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
+static GType gee_read_only_set_iterator_get_type (void);
+GeeIterator* gee_iterable_iterator (GeeIterable* self);
+static GeeIterator* gee_read_only_set_real_iterator (GeeIterable* base);
+gboolean gee_collection_contains (GeeCollection* self, gconstpointer item);
+static gboolean gee_read_only_set_real_contains (GeeCollection* base, gconstpointer item);
+static gboolean gee_read_only_set_real_add (GeeCollection* base, gconstpointer item);
+static gboolean gee_read_only_set_real_remove (GeeCollection* base, gconstpointer item);
+static void gee_read_only_set_real_clear (GeeCollection* base);
+static gboolean gee_read_only_set_real_add_all (GeeCollection* base, GeeCollection* collection);
+gboolean gee_iterator_next (GeeIterator* self);
+gpointer gee_iterator_get (GeeIterator* self);
+static gboolean gee_read_only_set_real_contains_all (GeeCollection* base, GeeCollection* collection);
+static gboolean gee_read_only_set_real_remove_all (GeeCollection* base, GeeCollection* collection);
+static gboolean gee_read_only_set_real_retain_all (GeeCollection* base, GeeCollection* collection);
+gpointer* gee_collection_to_array (GeeCollection* self, int* result_length1);
+static gpointer* gee_read_only_set_real_to_array (GeeCollection* base, int* result_length1);
+gint gee_collection_get_size (GeeCollection* self);
+gboolean gee_collection_get_is_empty (GeeCollection* self);
 #define GEE_READ_ONLY_SET_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_READ_ONLY_SET_TYPE_ITERATOR, GeeReadOnlySetIteratorPrivate))
 enum  {
 	GEE_READ_ONLY_SET_ITERATOR_DUMMY_PROPERTY,
@@ -82,16 +214,11 @@ enum  {
 };
 static gboolean gee_read_only_set_iterator_real_next (GeeIterator* base);
 static gpointer gee_read_only_set_iterator_real_get (GeeIterator* base);
-static GeeReadOnlySetIterator* gee_read_only_set_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
-static GeeReadOnlySetIterator* gee_read_only_set_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
-static gpointer gee_read_only_set_iterator_parent_class = NULL;
-static GeeIteratorIface* gee_read_only_set_iterator_gee_iterator_parent_iface = NULL;
-static GType gee_read_only_set_iterator_get_type (void);
-static gpointer gee_read_only_set_parent_class = NULL;
-static GeeIterableIface* gee_read_only_set_gee_iterable_parent_iface = NULL;
-static GeeCollectionIface* gee_read_only_set_gee_collection_parent_iface = NULL;
-static GeeSetIface* gee_read_only_set_gee_set_parent_iface = NULL;
+static void gee_read_only_set_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
+static void gee_read_only_set_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
 static void gee_read_only_set_finalize (GObject* obj);
+static void gee_read_only_set_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
+static void gee_read_only_set_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
 
 
 
@@ -99,8 +226,7 @@ GeeReadOnlySet* gee_read_only_set_construct (GType object_type, GType g_type, GB
 	GParameter * __params;
 	GParameter * __params_it;
 	GeeReadOnlySet * self;
-	g_return_val_if_fail (set == NULL || GEE_IS_SET (set), NULL);
-	__params = g_new0 (GParameter, 3);
+	__params = g_new0 (GParameter, 4);
 	__params_it = __params;
 	__params_it->name = "g-type";
 	g_value_init (&__params_it->value, G_TYPE_GTYPE);
@@ -114,8 +240,11 @@ GeeReadOnlySet* gee_read_only_set_construct (GType object_type, GType g_type, GB
 	g_value_init (&__params_it->value, G_TYPE_POINTER);
 	g_value_set_pointer (&__params_it->value, g_destroy_func);
 	__params_it++;
+	__params_it->name = "set";
+	g_value_init (&__params_it->value, GEE_TYPE_SET);
+	g_value_set_object (&__params_it->value, set);
+	__params_it++;
 	self = g_object_newv (object_type, __params_it - __params, __params);
-	gee_read_only_set_set_set (self, set);
 	while (__params_it > __params) {
 		--__params_it;
 		g_value_unset (&__params_it->value);
@@ -130,83 +259,176 @@ GeeReadOnlySet* gee_read_only_set_new (GType g_type, GBoxedCopyFunc g_dup_func,
 }
 
 
-static GType gee_read_only_set_real_get_element_type (GeeIterable* base) {
-	GeeReadOnlySet * self;
-	self = GEE_READ_ONLY_SET (base);
-	return self->priv->g_type;
-}
-
-
 static GeeIterator* gee_read_only_set_real_iterator (GeeIterable* base) {
 	GeeReadOnlySet * self;
-	self = GEE_READ_ONLY_SET (base);
+	GeeIterator* result;
+	self = (GeeReadOnlySet*) base;
 	if (self->priv->_set == NULL) {
-		return GEE_ITERATOR (gee_read_only_set_iterator_new (self->priv->g_type, ((GBoxedCopyFunc) (self->priv->g_dup_func)), self->priv->g_destroy_func));
+		result = (GeeIterator*) gee_read_only_set_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func);
+		return result;
 	}
-	return gee_iterable_iterator (GEE_ITERABLE (self->priv->_set));
+	result = gee_iterable_iterator ((GeeIterable*) self->priv->_set);
+	return result;
 }
 
 
 static gboolean gee_read_only_set_real_contains (GeeCollection* base, gconstpointer item) {
 	GeeReadOnlySet * self;
-	self = GEE_READ_ONLY_SET (base);
+	gboolean result;
+	self = (GeeReadOnlySet*) base;
 	if (self->priv->_set == NULL) {
-		return FALSE;
+		result = FALSE;
+		return result;
 	}
-	return gee_collection_contains (GEE_COLLECTION (self->priv->_set), item);
+	result = gee_collection_contains ((GeeCollection*) self->priv->_set, item);
+	return result;
 }
 
 
 static gboolean gee_read_only_set_real_add (GeeCollection* base, gconstpointer item) {
 	GeeReadOnlySet * self;
-	self = GEE_READ_ONLY_SET (base);
+	gboolean result;
+	self = (GeeReadOnlySet*) base;
 	g_assert_not_reached ();
 }
 
 
 static gboolean gee_read_only_set_real_remove (GeeCollection* base, gconstpointer item) {
 	GeeReadOnlySet * self;
-	self = GEE_READ_ONLY_SET (base);
+	gboolean result;
+	self = (GeeReadOnlySet*) base;
 	g_assert_not_reached ();
 }
 
 
 static void gee_read_only_set_real_clear (GeeCollection* base) {
 	GeeReadOnlySet * self;
-	self = GEE_READ_ONLY_SET (base);
+	self = (GeeReadOnlySet*) base;
 	g_assert_not_reached ();
 }
 
 
-static gint gee_read_only_set_real_get_size (GeeCollection* base) {
-	GeeReadOnlySet* self;
-	self = GEE_READ_ONLY_SET (base);
-	return gee_collection_get_size (GEE_COLLECTION (self->priv->_set));
+static gboolean gee_read_only_set_real_add_all (GeeCollection* base, GeeCollection* collection) {
+	GeeReadOnlySet * self;
+	gboolean result;
+	self = (GeeReadOnlySet*) base;
+	g_return_val_if_fail (collection != NULL, FALSE);
+	g_assert_not_reached ();
 }
 
 
-void gee_read_only_set_set_set (GeeReadOnlySet* self, GeeSet* value) {
-	GeeSet* _tmp1;
-	GeeSet* _tmp0;
-	g_return_if_fail (GEE_IS_READ_ONLY_SET (self));
-	_tmp1 = NULL;
-	_tmp0 = NULL;
-	self->priv->_set = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (self->priv->_set == NULL ? NULL : (self->priv->_set = (g_object_unref (self->priv->_set), NULL))), _tmp1);
-	g_object_notify (((GObject *) (self)), "set");
+static gboolean gee_read_only_set_real_contains_all (GeeCollection* base, GeeCollection* collection) {
+	GeeReadOnlySet * self;
+	gboolean result;
+	self = (GeeReadOnlySet*) base;
+	g_return_val_if_fail (collection != NULL, FALSE);
+	{
+		GeeIterator* _element_it;
+		_element_it = gee_iterable_iterator ((GeeIterable*) collection);
+		while (TRUE) {
+			gpointer element;
+			if (!gee_iterator_next (_element_it)) {
+				break;
+			}
+			element = gee_iterator_get (_element_it);
+			if (!gee_collection_contains ((GeeCollection*) self, element)) {
+				result = FALSE;
+				((element == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (element = (self->priv->g_destroy_func (element), NULL));
+				(_element_it == NULL) ? NULL : (_element_it = (g_object_unref (_element_it), NULL));
+				return result;
+			}
+			((element == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (element = (self->priv->g_destroy_func (element), NULL));
+		}
+		(_element_it == NULL) ? NULL : (_element_it = (g_object_unref (_element_it), NULL));
+	}
+	result = TRUE;
+	return result;
+}
+
+
+static gboolean gee_read_only_set_real_remove_all (GeeCollection* base, GeeCollection* collection) {
+	GeeReadOnlySet * self;
+	gboolean result;
+	self = (GeeReadOnlySet*) base;
+	g_return_val_if_fail (collection != NULL, FALSE);
+	g_assert_not_reached ();
+}
+
+
+static gboolean gee_read_only_set_real_retain_all (GeeCollection* base, GeeCollection* collection) {
+	GeeReadOnlySet * self;
+	gboolean result;
+	self = (GeeReadOnlySet*) base;
+	g_return_val_if_fail (collection != NULL, FALSE);
+	g_assert_not_reached ();
+}
+
+
+static gpointer* gee_read_only_set_real_to_array (GeeCollection* base, int* result_length1) {
+	GeeReadOnlySet * self;
+	gpointer* result;
+	gint _tmp0_;
+	gpointer* _tmp1_;
+	self = (GeeReadOnlySet*) base;
+	_tmp1_ = NULL;
+	result = (_tmp1_ = gee_collection_to_array ((GeeCollection*) self->priv->_set, &_tmp0_), *result_length1 = _tmp0_, _tmp1_);
+	return result;
+}
+
+
+static gint gee_read_only_set_real_get_size (GeeCollection* base) {
+	gint result;
+	GeeReadOnlySet* self;
+	self = (GeeReadOnlySet*) base;
+	result = gee_collection_get_size ((GeeCollection*) self->priv->_set);
+	return result;
+}
+
+
+static gboolean gee_read_only_set_real_get_is_empty (GeeCollection* base) {
+	gboolean result;
+	GeeReadOnlySet* self;
+	self = (GeeReadOnlySet*) base;
+	result = gee_collection_get_is_empty ((GeeCollection*) self->priv->_set);
+	return result;
+}
+
+
+static void gee_read_only_set_set_set (GeeReadOnlySet* self, GeeSet* value) {
+	GeeSet* _tmp1_;
+	GeeSet* _tmp0_;
+	g_return_if_fail (self != NULL);
+	_tmp1_ = NULL;
+	_tmp0_ = NULL;
+	self->priv->_set = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_set == NULL) ? NULL : (self->priv->_set = (g_object_unref (self->priv->_set), NULL)), _tmp1_);
+	g_object_notify ((GObject *) self, "set");
+}
+
+
+static GType gee_read_only_set_real_get_element_type (GeeIterable* base) {
+	GType result;
+	GeeReadOnlySet* self;
+	self = (GeeReadOnlySet*) base;
+	result = self->priv->g_type;
+	return result;
 }
 
 
 static gboolean gee_read_only_set_iterator_real_next (GeeIterator* base) {
 	GeeReadOnlySetIterator * self;
-	self = GEE_READ_ONLY_SET_ITERATOR (base);
-	return FALSE;
+	gboolean result;
+	self = (GeeReadOnlySetIterator*) base;
+	result = FALSE;
+	return result;
 }
 
 
 static gpointer gee_read_only_set_iterator_real_get (GeeIterator* base) {
 	GeeReadOnlySetIterator * self;
-	self = GEE_READ_ONLY_SET_ITERATOR (base);
-	return NULL;
+	gpointer result;
+	self = (GeeReadOnlySetIterator*) base;
+	result = NULL;
+	return result;
 }
 
 
@@ -243,37 +465,6 @@ static GeeReadOnlySetIterator* gee_read_only_set_iterator_new (GType g_type, GBo
 }
 
 
-static void gee_read_only_set_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
-	GeeReadOnlySetIterator * self;
-	self = GEE_READ_ONLY_SET_ITERATOR (object);
-	switch (property_id) {
-		default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-		break;
-	}
-}
-
-
-static void gee_read_only_set_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
-	GeeReadOnlySetIterator * self;
-	self = GEE_READ_ONLY_SET_ITERATOR (object);
-	switch (property_id) {
-		case GEE_READ_ONLY_SET_ITERATOR_G_TYPE:
-		self->priv->g_type = g_value_get_gtype (value);
-		break;
-		case GEE_READ_ONLY_SET_ITERATOR_G_DUP_FUNC:
-		self->priv->g_dup_func = g_value_get_pointer (value);
-		break;
-		case GEE_READ_ONLY_SET_ITERATOR_G_DESTROY_FUNC:
-		self->priv->g_destroy_func = g_value_get_pointer (value);
-		break;
-		default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-		break;
-	}
-}
-
-
 static void gee_read_only_set_iterator_class_init (GeeReadOnlySetIteratorClass * klass) {
 	gee_read_only_set_iterator_parent_class = g_type_class_peek_parent (klass);
 	g_type_class_add_private (klass, sizeof (GeeReadOnlySetIteratorPrivate));
@@ -309,13 +500,11 @@ static GType gee_read_only_set_iterator_get_type (void) {
 }
 
 
-static void gee_read_only_set_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
-	GeeReadOnlySet * self;
-	self = GEE_READ_ONLY_SET (object);
+static void gee_read_only_set_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
+	GeeReadOnlySetIterator * self;
+	gpointer boxed;
+	self = GEE_READ_ONLY_SET_ITERATOR (object);
 	switch (property_id) {
-		case GEE_READ_ONLY_SET_SIZE:
-		g_value_set_int (value, gee_collection_get_size (GEE_COLLECTION (self)));
-		break;
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -323,25 +512,22 @@ static void gee_read_only_set_get_property (GObject * object, guint property_id,
 }
 
 
-static void gee_read_only_set_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
-	GeeReadOnlySet * self;
-	self = GEE_READ_ONLY_SET (object);
+static void gee_read_only_set_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
+	GeeReadOnlySetIterator * self;
+	self = GEE_READ_ONLY_SET_ITERATOR (object);
 	switch (property_id) {
-		case GEE_READ_ONLY_SET_SET:
-		gee_read_only_set_set_set (self, g_value_get_object (value));
-		break;
-		case GEE_READ_ONLY_SET_G_TYPE:
-		self->priv->g_type = g_value_get_gtype (value);
-		break;
-		case GEE_READ_ONLY_SET_G_DUP_FUNC:
-		self->priv->g_dup_func = g_value_get_pointer (value);
-		break;
-		case GEE_READ_ONLY_SET_G_DESTROY_FUNC:
-		self->priv->g_destroy_func = g_value_get_pointer (value);
-		break;
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
+		case GEE_READ_ONLY_SET_ITERATOR_G_TYPE:
+		self->priv->g_type = g_value_get_gtype (value);
+		break;
+		case GEE_READ_ONLY_SET_ITERATOR_G_DUP_FUNC:
+		self->priv->g_dup_func = g_value_get_pointer (value);
+		break;
+		case GEE_READ_ONLY_SET_ITERATOR_G_DESTROY_FUNC:
+		self->priv->g_destroy_func = g_value_get_pointer (value);
+		break;
 	}
 }
 
@@ -356,14 +542,16 @@ static void gee_read_only_set_class_init (GeeReadOnlySetClass * klass) {
 	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SET_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SET_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_override_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SET_SIZE, "size");
-	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SET_SET, g_param_spec_object ("set", "set", "set", GEE_TYPE_SET, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE));
+	g_object_class_override_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SET_IS_EMPTY, "is-empty");
+	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SET_SET, g_param_spec_object ("set", "set", "set", GEE_TYPE_SET, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_override_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SET_ELEMENT_TYPE, "element-type");
 }
 
 
 static void gee_read_only_set_gee_iterable_interface_init (GeeIterableIface * iface) {
 	gee_read_only_set_gee_iterable_parent_iface = g_type_interface_peek_parent (iface);
-	iface->get_element_type = gee_read_only_set_real_get_element_type;
 	iface->iterator = gee_read_only_set_real_iterator;
+	iface->get_element_type = gee_read_only_set_real_get_element_type;
 }
 
 
@@ -373,7 +561,13 @@ static void gee_read_only_set_gee_collection_interface_init (GeeCollectionIface
 	iface->add = gee_read_only_set_real_add;
 	iface->remove = gee_read_only_set_real_remove;
 	iface->clear = gee_read_only_set_real_clear;
+	iface->add_all = gee_read_only_set_real_add_all;
+	iface->contains_all = gee_read_only_set_real_contains_all;
+	iface->remove_all = gee_read_only_set_real_remove_all;
+	iface->retain_all = gee_read_only_set_real_retain_all;
+	iface->to_array = gee_read_only_set_real_to_array;
 	iface->get_size = gee_read_only_set_real_get_size;
+	iface->get_is_empty = gee_read_only_set_real_get_is_empty;
 }
 
 
@@ -390,7 +584,7 @@ static void gee_read_only_set_instance_init (GeeReadOnlySet * self) {
 static void gee_read_only_set_finalize (GObject* obj) {
 	GeeReadOnlySet * self;
 	self = GEE_READ_ONLY_SET (obj);
-	(self->priv->_set == NULL ? NULL : (self->priv->_set = (g_object_unref (self->priv->_set), NULL)));
+	(self->priv->_set == NULL) ? NULL : (self->priv->_set = (g_object_unref (self->priv->_set), NULL));
 	G_OBJECT_CLASS (gee_read_only_set_parent_class)->finalize (obj);
 }
 
@@ -411,5 +605,49 @@ GType gee_read_only_set_get_type (void) {
 }
 
 
+static void gee_read_only_set_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
+	GeeReadOnlySet * self;
+	gpointer boxed;
+	self = GEE_READ_ONLY_SET (object);
+	switch (property_id) {
+		case GEE_READ_ONLY_SET_SIZE:
+		g_value_set_int (value, gee_collection_get_size ((GeeCollection*) self));
+		break;
+		case GEE_READ_ONLY_SET_IS_EMPTY:
+		g_value_set_boolean (value, gee_collection_get_is_empty ((GeeCollection*) self));
+		break;
+		case GEE_READ_ONLY_SET_ELEMENT_TYPE:
+		g_value_set_gtype (value, gee_iterable_get_element_type ((GeeIterable*) self));
+		break;
+		default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
+
+static void gee_read_only_set_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
+	GeeReadOnlySet * self;
+	self = GEE_READ_ONLY_SET (object);
+	switch (property_id) {
+		case GEE_READ_ONLY_SET_SET:
+		gee_read_only_set_set_set (self, g_value_get_object (value));
+		break;
+		default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+		case GEE_READ_ONLY_SET_G_TYPE:
+		self->priv->g_type = g_value_get_gtype (value);
+		break;
+		case GEE_READ_ONLY_SET_G_DUP_FUNC:
+		self->priv->g_dup_func = g_value_get_pointer (value);
+		break;
+		case GEE_READ_ONLY_SET_G_DESTROY_FUNC:
+		self->priv->g_destroy_func = g_value_get_pointer (value);
+		break;
+	}
+}
+
+
 
 
